@@ -1,14 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import division
 
 import datetime
 
 from sql import Null
 
-from trytond.model import ModelView, ModelSQL, fields, sequence_ordered
+from trytond.model import ModelView, ModelSQL, fields, sequence_ordered, tree
 from trytond.pyson import Eval
-from trytond import backend
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.tools import reduce_ids, grouped_slice
@@ -16,7 +14,7 @@ from trytond.tools import reduce_ids, grouped_slice
 __all__ = ['Work']
 
 
-class Work(sequence_ordered(), ModelSQL, ModelView):
+class Work(sequence_ordered(), tree(separator='\\'), ModelSQL, ModelView):
     'Work Effort'
     __name__ = 'project.work'
     name = fields.Char('Name', required=True, select=True)
@@ -121,15 +119,11 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
     @classmethod
     def __register__(cls, module_name):
         TimesheetWork = Pool().get('timesheet.work')
-        TableHandler = backend.get('TableHandler')
         cursor = Transaction().connection.cursor()
-        table_project_work = TableHandler(cls, module_name)
-        table_timesheet_work = TableHandler(TimesheetWork, module_name)
+        table_project_work = cls.__table_handler__(module_name)
         project = cls.__table__()
         timesheet = TimesheetWork.__table__()
 
-        migrate_sequence = (not table_project_work.column_exist('sequence')
-            and table_timesheet_work.column_exist('sequence'))
         work_exist = table_project_work.column_exist('work')
         add_parent = (not table_project_work.column_exist('parent')
             and work_exist)
@@ -140,20 +134,6 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
 
         super(Work, cls).__register__(module_name)
 
-        # Migration from 2.0: copy sequence from timesheet to project
-        if migrate_sequence:
-            cursor.execute(*timesheet.join(project,
-                    condition=project.work == timesheet.id
-                    ).select(timesheet.sequence, timesheet.id))
-            for sequence, id_ in cursor.fetchall():
-                cursor.execute(*project.update(
-                        columns=[project.sequence],
-                        values=[sequence],
-                        where=project.work == id_))
-
-        # Migration from 2.4: drop required on sequence
-        table_project_work.not_null_action('sequence', action='remove')
-
         # Migration from 3.4: change effort into timedelta effort_duration
         if table_project_work.column_exist('effort'):
             cursor.execute(*project.select(project.id, project.effort,
@@ -213,7 +193,7 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
                         [timesheet.origin, timesheet.name],
                         ['%s,%s' % (cls.__name__, project_id), Null],
                         where=timesheet.id == work_id))
-            table_project_work.drop_column('work', exception=True)
+            table_project_work.drop_column('work')
 
     @classmethod
     def __setup__(cls):
@@ -226,10 +206,16 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
                     'opened.'),
                 })
 
+    @classmethod
+    def index_set_field(cls, name):
+        index = super(Work, cls).index_set_field(name)
+        if name in {'timesheet_start_date', 'timesheet_end_date'}:
+            index = cls.index_set_field('timesheet_available') + 1
+        return index
+
     @classmethod
     def validate(cls, works):
         super(Work, cls).validate(works)
-        cls.check_recursion(works, rec_name='name')
         for work in works:
             work.check_state()
 
@@ -266,27 +252,6 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
             return 0
         return self.timesheet_duration.total_seconds() / 60 / 60
 
-    def get_rec_name(self, name):
-        if self.parent:
-            return self.parent.get_rec_name(name) + '\\' + self.name
-        else:
-            return self.name
-
-    @classmethod
-    def search_rec_name(cls, name, clause):
-        if isinstance(clause[2], basestring):
-            values = clause[2].split('\\')
-            values.reverse()
-            domain = []
-            field = 'name'
-            for name in values:
-                domain.append((field, clause[1], name.strip()))
-                field = 'parent.' + field
-        else:
-            domain = [('name',) + tuple(clause[1:])]
-        ids = [w.id for w in cls.search(domain, order=[])]
-        return [('parent', 'child_of', ids)]
-
     @classmethod
     def default_timesheet_available(cls):
         return False
@@ -337,7 +302,7 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
     def sum_tree(cls, works, values, parents):
         result = values.copy()
         works = set((w.id for w in works))
-        leafs = works - set(parents.itervalues())
+        leafs = works - set(parents.values())
         while leafs:
             for work in leafs:
                 works.remove(work)
@@ -415,19 +380,10 @@ class Work(sequence_ordered(), ModelSQL, ModelView):
     def copy(cls, project_works, default=None):
         if default is None:
             default = {}
-
-        timesheet_default = default.copy()
-        for key in timesheet_default.keys():
-            if key in cls._fields:
-                del timesheet_default[key]
-        timesheet_default['children'] = None
-        new_project_works = []
-        for project_work in project_works:
-            pwdefault = default.copy()
-            pwdefault['children'] = None
-            new_project_works.extend(super(Work, cls).copy([project_work],
-                    default=pwdefault))
-        return new_project_works
+        else:
+            default = default.copy()
+        default.setdefault('children', None)
+        return super().copy(project_works, default=default)
 
     @classmethod
     def delete(cls, project_works):
